import { ref, onScopeDispose, watch, nextTick, unref } from 'vue'
import type { Ref } from 'vue'
import { isDom } from '@/utils/common'
import TintImageWorker from '@/utils/image.worker?worker'

export type RGBA = number[]

export type Point = [number, number]
export interface SetImgBgOption {
  image?: string | HTMLImageElement | null
  basePoint?: Point[]
  distance?: number
  enableSharpen?: boolean
  rotate?: number
  scale?: number
}

interface InitOption {
  image: string | HTMLImageElement
}

export const initImage = async ({ image }: InitOption) => {
  if (!image) {
    return console.warn(`return for img define ${image}`)
  }

  if (!isDom<HTMLImageElement>(image)) {
    image = await new Promise<HTMLImageElement>((resolve, reject) => {
      const img = new Image()
      img.src = image as string
      img.onload = () => {
        resolve(img)
      }
      img.onerror = reject
    })
  }
  return image
}

interface MessageData {
  drawing: boolean
  blob?: Blob
}

export const useSetImgBg = (option: Ref<SetImgBgOption>) => {
  const imageWorker = new TintImageWorker()
  const drawing = ref<boolean>(false)
  const dataUrl = ref('')

  imageWorker.addEventListener('message', (e) => {
    const data = e.data as MessageData
    drawing.value = data.drawing
    dataUrl.value = data.blob ? URL.createObjectURL(data.blob) : ''
  })

  watch(
    [
      () => option.value?.image,
      () => option.value?.distance,
      () => option.value?.basePoint,
      () => option.value?.enableSharpen,
    ],
    () => {
      nextTick(() => {
        const opt = unref(option)
        if (!opt.image) {
          drawing.value = false
          dataUrl.value = ''
          return
        }
        initImage({ image: opt.image }).then((image) => {
          if (!image) return
          createImageBitmap(image).then((imageBitMap) => {
            imageWorker.postMessage(
              {
                type: 'init',
                imageBitMap,
                basePoint: opt.basePoint,
                enableSharpen: opt.enableSharpen,
                distance: opt.distance,
                scale: opt.scale,
                rotate: opt.rotate,
              },
              [imageBitMap]
            )
          })
        })
      })
    },
    { immediate: true }
  )

  watch(
    () => option.value.scale,
    () => {
      imageWorker.postMessage({ type: 'scale', scale: option.value?.scale })
    }
  )

  watch(
    () => option.value.rotate,
    () => {
      imageWorker.postMessage({ type: 'rotate', rotate: option.value?.rotate })
    }
  )

  watch(dataUrl, (current, prev) => {
    if (prev) {
      try {
        URL.revokeObjectURL(prev)
      } catch (error) {}
    }
  })

  onScopeDispose(() => {
    if (dataUrl.value) {
      try {
        URL.revokeObjectURL(dataUrl.value)
      } catch (error) {}
    }
    imageWorker.terminate()
  })

  return {
    drawing,
    dataUrl,
  }
}
